/*
 * Created by haozhicao@tencent.com on 2021/12/14.
 * popup_layout.dart
 * 计算Menu在屏幕中的位置
 */

import 'package:flutter/material.dart';

/// 菜单在屏幕的Padding
const double kMenuScreenPadding = 8.0;

/// 计算Menu在屏幕中的位置
class PopupMenuRouteLayout extends SingleChildLayoutDelegate {
  PopupMenuRouteLayout(this.position, this.selectedItemOffset,
      this.textDirection, this.width, this.menuWidth, this.height);

  // 底层按钮的矩形，相对于叠加层的尺寸。
  final RelativeRect position;

  // 菜单顶部到所选项目中间的距离。
  // 如果没有Item以这种方式定位，这将为null
  final double selectedItemOffset;

  // 向左还是向右
  final TextDirection textDirection;

  final double width;
  final double height;
  final double menuWidth;


  @override
  BoxConstraints getConstraintsForChild(BoxConstraints constraints) {
    // Menu在每个方向最多可以是叠加层的大小减去 8.0
    return BoxConstraints.loose(Size(constraints.biggest.width - kMenuScreenPadding * 2.0, constraints.biggest.height - kMenuScreenPadding * 2.0));
  }

  @override
  Offset getPositionForChild(Size size, Size childSize) {
    double y;
    if (selectedItemOffset.isInfinite) {
      y = position.top;
    } else {
      y = position.top +
          (size.height - position.top - position.bottom) / 2.0 -
          selectedItemOffset;
    }

    double x;

    // 如果menu 的宽度 小于 child 的宽度，则直接把menu 放在 child 中间
    if (childSize.width < width) {
      x = position.left + (width - childSize.width) / 2;
    } else {
      // 如果靠右
      if (position.left > size.width - (position.left + width)) {
        if (size.width - (position.left + width) >
            childSize.width / 2 + kMenuScreenPadding) {
          x = position.left - (childSize.width - width) / 2;
        } else {
          x = position.left + width - childSize.width;
        }
      } else if (position.left < size.width - (position.left + width)) {
        if (position.left > childSize.width / 2 + kMenuScreenPadding) {
          x = position.left - (childSize.width - width) / 2;
        } else
          x = position.left;
      } else {
        x = position.right - width / 2 - childSize.width / 2;
      }
    }

    if (y < kMenuScreenPadding)
      y = kMenuScreenPadding;
    else if (y + childSize.height > size.height - kMenuScreenPadding)
      y = size.height - childSize.height;
    else if (y < childSize.height * 2) {
      y = position.top + height;
    }
    return Offset(x, y);
  }

  @override
  bool shouldRelayout(PopupMenuRouteLayout oldDelegate) {
    return position != oldDelegate.position;
  }
}